/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::units
 * @author     David Schiebener <schiebener at kit dot edu>
 * @date       2014
 * @copyright  http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_ROBOTAPI_HAND_UNIT_OBSERVER_H
#define _ARMARX_ROBOTAPI_HAND_UNIT_OBSERVER_H

#include <RobotAPI/interface/units/HandUnitInterface.h>
#include <Core/observers/Observer.h>


namespace armarx
{

    class HandUnitObserverPropertyDefinitions:
            public ComponentPropertyDefinitions
    {
    public:
        HandUnitObserverPropertyDefinitions(std::string prefix):
            ComponentPropertyDefinitions(prefix)
        {
            //defineRequiredProperty<std::string>("HapticTopicName", "Name of the HapticUnit Topic");
        }
    };


    // TODO !!!
    /**
     * @class HandUnitObserver
     * @ingroup RobotAPI-SensorActorUnits-observers
     * @brief The HandUnitObserver class
     */
    class HandUnitObserver :
            virtual public Observer,
            virtual public HandUnitObserverInterface
    {
    public:
        HandUnitObserver(){}

        // framework hooks
        virtual std::string getDefaultName() const { return "HandUnitObserver"; }
        virtual void onInitObserver();
        virtual void onConnectObserver();
        virtual void onExitObserver();

        // observer interface
        // implementation: todo...
        virtual void reportHandShaped(const std::string &handName, const std::string &shapeName, const ::Ice::Current& = ::Ice::Current());
        virtual void reportNewHandShapeName(const std::string &handName, const ::std::string& handShapeName, const ::Ice::Current& = ::Ice::Current());
        void reportJointAngles(const NameValueMap &actualJointAngles, const Ice::Current& = ::Ice::Current());
        void reportJointPressures(const NameValueMap &actualJointPressures, const Ice::Current& = ::Ice::Current());

        /**
         * @see PropertyUser::createPropertyDefinitions()
         */
        //virtual PropertyDefinitionsPtr createPropertyDefinitions();
    private:


    };
}

#endif
