/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarX::Gui
* @author     Mirko Waechter ( mirko.waechter at kit dot edu)
* @date       2012
* @copyright  http://www.gnu.org/licenses/gpl.txt
*             GNU General Public License
*/

#ifndef ARMARXPLOTTER_H
#define ARMARXPLOTTER_H

#include "ui_ArmarXPlotter.h"

// ArmarX
#include <Gui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <Gui/gui-plugins/ObserverPropertiesPlugin/ObserverItemModel.h>

#include <Core/core/Component.h>
#include <Core/core/system/ImportExportComponent.h>
#include <Core/core/system/AbstractFactoryMethod.h>
#include <Core/interface/observers/ConditionHandlerInterface.h>
#include <Core/observers/variant/DataFieldIdentifier.h>
#include <Core/observers/variant/Variant.h>
#include <Core/util/json/JSONObject.h>
#include <Core/core/services/tasks/PeriodicTask.h>

// QT
#include <QWidget>
#include <QDialog>
#include <QDateTime>

#include <vector>
#include <fstream>

//forward declarations
class QwtPlotCurve;

namespace armarx
{
    struct TimeData{
        TimeData(const IceUtil::Time &time, const VariantPtr &data): time(time), data(data){}

        IceUtil::Time time;
        VariantPtr data;
    };

    typedef std::map< std::string, ObserverInterfacePrx> ProxyMap;
    typedef std::map< std::string, std::vector<TimeData> > GraphDataMap;

    class ArmarXPlotterDialog;
    /*!
     * \class ArmarXPlotter
     * \brief The plotter widget allows the user to plot any sensor channel.
     * \image html PlotterGUI.png
     * A sensor channel can be selected for plotting by clicking the wrench button at the bottom.
     * \ingroup RobotAPI-ArmarXGuiPlugins ArmarXGuiPlugins
      */
    class ARMARXCOMPONENT_IMPORT_EXPORT
    ArmarXPlotter:
            public ArmarXComponentWidgetController
    {
        Q_OBJECT
    public:


        std::map<std::string, QwtPlotCurve *> curves;
        Ui::ArmarXPlotter ui;
        QPointer<ArmarXPlotterDialog> dialog;
        QTimer * timer;
        ConditionHandlerInterfacePrx handler;

        // Configuration Parameters
        int updateInterval;
        int shownInterval;
        QStringList selectedChannels;
        QString loggingDir;

        explicit ArmarXPlotter();
        ~ArmarXPlotter();
        //inherited from ArmarXWidget
        virtual QString getWidgetName() const { return "Observers.Plotter";}
        void loadSettings(QSettings * settings);
        void saveSettings(QSettings * settings);
        //for AbstractFactoryMethod class


        // inherited from Component
        virtual void onInitComponent();
        virtual void onConnectComponent();
        virtual void onExitComponent();



        /**
        * emits the closeRequest signal
        */
        void onCloseWidget(QCloseEvent *event);
        QwtPlotCurve * createCurve(const QString &label);
    signals:

    public slots:
        void ButtonAddSensorChannelClicked();
        void configDialog();
        void updateGraph();
        void showCurve(QwtPlotItem *item, bool on);
        void autoScale(bool toggled);
        void plottingPaused(bool toggled);
        void configDone();
        void toggleLogging(bool toggled);

    private slots:
        void setupCurves();
    protected:
        boost::shared_ptr<QSettings> settings;
    private:

        void pollingExec();
        QDateTime startUpTime;
        GraphDataMap dataMap;
        std::map< std::string, ObserverInterfacePrx> proxyMap;
        bool __plottingPaused;
        JSONObjectPtr json;


        std::map<std::string, VariantPtr> getData(const QStringList &channels, GraphDataMap &dataMaptoAppend);
        void logToFile(const IceUtil::Time &time,const std::map<std::string, VariantPtr> &dataMaptoAppend);

        Mutex dataMutex;
        PeriodicTask<ArmarXPlotter>::pointer_type pollingTask;
        int pollingInterval;
        std::ofstream logstream;
        IceUtil::Time logStartTime;
    };

}

#endif
