/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @package    ArmarX::RobotAPI
* @author     Stefan Ulbrich, Nikolaus Vahrenkamp
* @copyright  2011 Humanoids Group, HIS, KIT
* @license    http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#ifndef _ARMARX_ROBOTAPI_POSE_SLICE_
#define _ARMARX_ROBOTAPI_POSE_SLICE_

#include <Core/interface/observers/VariantBase.ice>
#include <Core/interface/observers/Filters.ice>

module armarx
{

    class Vector2Base extends VariantDataClass
    {
        float x = zeroInt;
        float y = zeroInt;
    };

    class Vector3Base extends VariantDataClass
    {
        float x = zeroInt;
        float y = zeroInt;
        float z = zeroInt;
    };

    sequence<Vector3Base> Vector3BaseList;

    class QuaternionBase extends VariantDataClass
    {
        float qw = zeroInt;
        float qx = zeroInt;
        float qy = zeroInt;
        float qz = zeroInt;
    };

    ["cpp:virtual"]
    class PoseBase extends VariantDataClass
    {
        Vector3Base position;
        QuaternionBase orientation;
    };

    sequence<PoseBase> PoseBaseList;

};

#endif
