/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Mirko Waechter <waechter at kit dot edu>
 * @copyright  2013 Mirko Waechter
 * @license    http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_ROBOTAPI_UNITS_FORCETORQUE_SLICE_
#define _ARMARX_ROBOTAPI_UNITS_FORCETORQUE_SLICE_

#include <RobotAPI/interface/units/UnitInterface.ice>
#include <RobotAPI/interface/core/RobotState.ice>
#include <RobotAPI/interface/core/FramedPoseBase.ice>

#include <Core/interface/core/UserException.ice>
#include <Core/interface/core/BasicTypes.ice>
#include <Core/interface/observers/VariantBase.ice>
#include <Core/interface/observers/ObserverInterface.ice>

module armarx
{
    interface ForceTorqueUnitInterface extends armarx::SensorActorUnitInterface
    {
        void setOffset(FramedDirectionBase forceOffsets, FramedDirectionBase torqueOffsets);
        void setToNull();
    };

    interface ForceTorqueUnitListener
    {
		/*!
			\param sensorNodeName The RobotNode on which the sensor is mounted.
			\param forces Set to zero if not available.
			\param torques Set to zero if not available.
		*/
        void reportSensorValues(string sensorNodeName, FramedDirectionBase forces, FramedDirectionBase torques);
    };

    interface ForceTorqueUnitObserverInterface extends ObserverInterface, ForceTorqueUnitListener
    {
        DatafieldRefBase getForceDatafield(string nodeName) throws UserException;
        DatafieldRefBase getTorqueDatafield(string nodeName) throws UserException;

        DatafieldRefBase createNulledDatafield(DatafieldRefBase forceTorqueDatafieldRef);
    };
};

#endif
