/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    ArmarX::Core
 * @author     Manfred Kroehnert (manfred dot kroehnert at kit dot edu)
 * @copyright  2013 Manfred Kroehnert
 * @license    http://www.gnu.org/licenses/gpl.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_CORE_UNITS_HANDUNIT_SLICE_
#define _ARMARX_CORE_UNITS_HANDUNIT_SLICE_

#include <RobotAPI/interface/units/UnitInterface.ice>
#include <RobotAPI/interface/units/KinematicUnitInterface.ice>

#include <Core/interface/core/UserException.ice>
#include <Core/interface/core/BasicTypes.ice>
#include <Core/interface/observers/VariantContainers.ice>
#include <Core/interface/observers/ObserverInterface.ice>


module armarx
{

    interface HandUnitInterface extends SensorActorUnitInterface
    {
        void setShape(string shapeName);
        SingleTypeVariantListBase getShapeNames();
        NameValueMap getShapeJointValues(string shapeName);
        string getHandName();


        /*!
         * \brief Inform the hand unit that an object has been successfully grasped.
         * \param objectName The name of the object
        *  E.g. a state that verifies if a grasp was successful can call this routine.
        */
        void setObjectGrasped(string objectName);

        /*!
         * \brief setObjectReleased Inform the hand unit that an object has been released.
         * \param objectName The name of the object
         */
        void setObjectReleased(string objectName);

        void setJointAngles(NameValueMap targetJointAngles);
    };

    interface HandUnitListener
    {
        void reportHandShaped(string handName, string handShapeName);
        void reportNewHandShapeName(string handName, string handShapeName);
        void reportJointAngles(NameValueMap actualJointAngles);
        void reportJointPressures(NameValueMap actualJointPressures);

    };

    interface HandUnitObserverInterface extends ObserverInterface, HandUnitListener
    {
    };

};

#endif
