/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Markus Grotz <markus dot grotz at kit dot edu>
 * @date       2015
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_ROBOTAPI_UNITS_INERTIALMEASUREMENTUNIT_SLICE_
#define _ARMARX_ROBOTAPI_UNITS_INERTIALMEASUREMENTUNIT_SLICE_


#include <RobotAPI/interface/units/UnitInterface.ice>
#include <RobotAPI/interface/core/RobotState.ice>

#include <Core/interface/core/UserException.ice>
#include <Core/interface/core/BasicTypes.ice>
#include <Core/interface/observers/VariantBase.ice>
#include <Core/interface/observers/Matrix.ice>
#include <Core/interface/observers/Timestamp.ice>
#include <Core/interface/observers/ObserverInterface.ice>



module armarx
{
    struct IMUData {
        FloatSequence orientationQuaternion;
        FloatSequence magneticRotation;
        FloatSequence gyroscopeRotation; 
        FloatSequence acceleration;
    };

    interface InertialMeasurementUnitInterface extends armarx::SensorActorUnitInterface
    {
    };

    interface InertialMeasurementUnitListener
    {
        void reportSensorValues(string device, string name, IMUData values, TimestampBase timestamp);
    };

    interface InertialMeasurementUnitObserverInterface extends ObserverInterface, InertialMeasurementUnitListener
    {
    };

};

#endif
